@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2001 Secure Software Solutions
+ * Copyright (c) 2001-2002 Secure Software, Inc
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,15 +24,34 @@
 #ifdef _MSC_VER
 #define strcasecmp  _stricmp
 #define strncasecmp _strnicmp
+#define lstat(x,y) _stat(x,y)
+#define PATH_MAX _MAX_PATH
+#include <windows.h>
+
+/* S_ISDIR does not exist in VC so we have to pull the macros from <sys/stat.h>
+ * of a reasonable OS  -- Robert */
+#define	_S_ISTYPE(mode, mask)	(((mode) & _S_IFMT) == (mask))
+#define	S_ISDIR(mode)	 _S_ISTYPE((mode), _S_IFDIR)
+#define _S_ISDIR(mode) S_ISDIR(mode)
+#define S_ISREG(m)      _S_ISTYPE((m), _S_IFREG)
+
+#else
+#include <sys/types.h>
+#include <dirent.h>
 #endif
 
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 #include <string.h>
 #include "tokens.h"
 #include "report.h"
 
+
+#include <unistd.h>
+#include <errno.h>
+
 lexer_t         ratslexer;
 Hash            database = (Hash)NULL;
 Hash	        defaultdb = (Hash)NULL;
@@ -47,7 +66,7 @@ static ignore_t *           current_ignore = (ignore_t *)NULL;
 static toctou_t *           toctous = (toctou_t *)NULL;
 static accumulator_t *      accumulators = (accumulator_t *)NULL;
 
-static void analyze_variable(void);
+static void analyze_variable(int token);
 static void analyze_identifier(void);
 static void analyze_comment(void);
 static void analyze_backticks(void);
@@ -62,7 +81,7 @@ int accumulate_text(accumulator_t *acc, char *text)
     length = strlen(text);
     if (*(acc->text) == (char *)NULL)
     {
-        *(acc->text) = (char *)malloc(length + 1);
+        *(acc->text) = (char *)calloc(length + 1,1);
         acc->length  = 0;
     }
     else
@@ -84,7 +103,7 @@ void push_accumulator(char **text)
 {
     accumulator_t * acc;
 
-    acc = (accumulator_t *)malloc(sizeof(accumulator_t));
+    acc = (accumulator_t *)calloc(sizeof(accumulator_t),1);
     acc->text    = text;
     acc->length  = (*text == (char *)NULL ? 0 : strlen(*text));
     acc->next    = accumulators;
@@ -139,11 +158,12 @@ int get_token(void)
         ungotten_token = -1;
     }
     else
-    {
-        if (!(token = (ratslexer.yylex)() ))
+      {
+      if (!(token = (ratslexer.yylex)() )) {
             return 0;
-        if (accumulators != (accumulator_t *)NULL)
-            accumulate_text(accumulators, *ratslexer.yytext);
+      }
+      if (accumulators != (accumulator_t *)NULL)
+	accumulate_text(accumulators, *ratslexer.yytext);
     }
 
     switch (token)
@@ -166,7 +186,7 @@ void scan_tokens(processorfn_t fn, void *arg)
 
     while ((token = get_token()) != 0)
     {
-        if (token != COMMENT)
+        if (token != TOKEN_COMMENT)
         {
             last_text_line = *ratslexer.lex_lineno;
             if (current_ignore != (ignore_t *)NULL)
@@ -178,18 +198,27 @@ void scan_tokens(processorfn_t fn, void *arg)
         if (fn != NULL)
             if (!fn(token, arg))
                 break;
-
         switch (token)
         {
-            case CHAR:
-                analyze_variable();
+        
+            case TOKEN_DOUBLE:
+            case TOKEN_FLOAT:
+            case TOKEN_INT:
+            case TOKEN_LONG:
+            case TOKEN_SHORT:
+            case TOKEN_STRUCT:
+            case TOKEN_VOID:
+            case TOKEN_ENUM:
+            case TOKEN_UNION:
+            case TOKEN_CHAR:
+                analyze_variable(token);
                 break;
 
-            case IDENTIFIER:
+            case TOKEN_IDENTIFIER:
                 analyze_identifier();
                 break;
 
-            case COMMENT:
+            case TOKEN_COMMENT:
                 analyze_comment();
                 break;
 
@@ -202,7 +231,7 @@ void scan_tokens(processorfn_t fn, void *arg)
 }
 
 static
-int gobble_backtick(int token, void *arg)
+ int gobble_backtick(int token, void *arg)
 {
     if (token == '`')
         return 0;
@@ -213,6 +242,7 @@ static void
 analyze_backticks(void)
 {
     int myline = *ratslexer.lex_lineno;
+    int column = *ratslexer.lex_column;
 
     if (ratslexer.lang == LANG_C)  
         return;
@@ -226,13 +256,13 @@ analyze_backticks(void)
     switch (ratslexer.lang)
     {
         case LANG_PYTHON:
-            log_pythonbacktick(myline, Medium);
+            log_pythonbacktick(myline, column, Medium);
             break;
         case LANG_PHP:    
-            log_phpbacktick(myline, Medium);
+            log_phpbacktick(myline, column,Medium);
             break;
         case LANG_PERL:
-            log_perlbacktick(myline, Medium);
+            log_perlbacktick(myline, column,Medium);
             break;
     }
 }
@@ -250,16 +280,19 @@ int check_buffer(int token, void *arg)
 
     if (token == '=')
         data->skip = 1;
-    else if (token == '[' && data->last_token == IDENTIFIER)
+    else if (token == '[' && data->last_token == TOKEN_IDENTIFIER)
         data->depth = depths[DEPTH_BRACE];
     else if (data->last_token == '[')
     {
         if (token != ']')
         {
-            if (data->depth)
-                log_staticbuffer(StaticLocalBuffer, data->lineno, High);
-            else
-                log_staticbuffer(StaticGlobalBuffer, data->lineno, Low);
+            if (data->initial_type == TOKEN_CHAR || (flags & ALL_STATIC))
+            {
+              if (data->depth)
+                  log_staticbuffer(StaticLocalBuffer, data->lineno, data->column,High);
+              else
+                  log_staticbuffer(StaticGlobalBuffer, data->lineno, data->column,Low);
+            }
             data->skip = 1;
         }
     }
@@ -273,7 +306,7 @@ int check_buffer(int token, void *arg)
  *      pointer initializers, i.e. char *foo = bar[15]; - MMessier, 09-May-2001
  */
 static
-void analyze_variable(void)
+void analyze_variable(int token)
 {
     charscan_t  data;
 
@@ -285,8 +318,10 @@ void analyze_variable(void)
     if (depths[DEPTH_PARENTHESIS] || depths[DEPTH_BRACKET])
         return;
 
+    data.column = *ratslexer.lex_column;
     data.lineno = *ratslexer.lex_lineno;
-    data.last_token = CHAR;
+    data.initial_type = token;
+    data.last_token = token;
     data.depth = depths[DEPTH_BRACE];
     data.skip = 0;
     scan_tokens(check_buffer, (void *)&data);
@@ -501,7 +536,7 @@ int check_argument(int token, void *arg)
 
     if (data->current == (argument_t *)NULL)
     {
-        data->current = (argument_t *)malloc(sizeof(argument_t));
+        data->current = (argument_t *)calloc(sizeof(argument_t),1);
         data->current->is_constant = 1;
         data->current->contains_ps = 0;
         data->current->yytext      = (char *)NULL;
@@ -514,7 +549,7 @@ int check_argument(int token, void *arg)
     {
         if (token < CONST_START || token > CONST_END)
             data->current->is_constant = 0;
-        else if (token == STRING_CONST && !data->current->contains_ps)
+        else if (token == TOKEN_STRING_CONST && !data->current->contains_ps)
             data->current->contains_ps = check_format_string(*ratslexer.yytext, -1);
     }
 
@@ -560,7 +595,7 @@ Vuln_t *locate_identifier(char *identifier)
 }
 
 static
-int push_identifier(char *identifier, int lineno)
+int push_identifier(char *identifier, int lineno, int column)
 {
     Vuln_t *        data;
     rats_stack_t *  frame;
@@ -568,10 +603,11 @@ int push_identifier(char *identifier, int lineno)
     if (database == NULL || (data = locate_identifier(identifier)) == NULL)
         return 0;
 
-    frame = (rats_stack_t *)malloc(sizeof(rats_stack_t));
+    frame = (rats_stack_t *)calloc(sizeof(rats_stack_t),1);
     frame->identifier = identifier;
     frame->data       = data;
     frame->lineno     = lineno;
+    frame->column     = column;
     frame->argc       = 0;
     frame->argv       = (argument_t *)NULL;
     frame->next       = current_frame;
@@ -602,7 +638,7 @@ void pop_identifier(void)
 }
 
 static
-void record_toctou(int argn, int lineno, int use)
+void record_toctou(int argn, int lineno, int use, int column)
 {
     toctou_t *      toctou;
     argument_t *    arg;
@@ -610,11 +646,12 @@ void record_toctou(int argn, int lineno, int use)
     if ((arg = get_argument(argn)) == (argument_t *)NULL)
         return;
 
-    toctou = (toctou_t *)malloc(sizeof(toctou_t));
+    toctou = (toctou_t *)calloc(sizeof(toctou_t),1);
     toctou->lineno = lineno;
     toctou->data   = current_frame->data;
     toctou->key    = strdup(arg->yytext);
     toctou->use    = use;
+    toctou->column = column;
     toctou->next   = toctous;
     toctous        = toctou;
     toctou_count++;
@@ -623,22 +660,34 @@ void record_toctou(int argn, int lineno, int use)
 static
 void analyze_identifier(void)
 {
-    int             analyzed, lineno, token;
+    int             to_analyze, analyzed, lineno, token, column;
     char *          identifier;
     Vuln_t *        data;
     argument_t *    arg;
 
+    to_analyze=token=0;
+
     lineno     = *ratslexer.lex_lineno;
+    column     = *ratslexer.lex_column;
     identifier = strdup(*ratslexer.yytext);
 
-    if (!push_identifier(identifier, lineno))
+
+    if (!push_identifier(identifier, lineno, column))
     {
         free(identifier);
         return;
     }
 
+    /* Special handling for cin in C++.  This is rather hackish -- Robert */
+    if(!strcmp(identifier,"cin")) {
+      if(token==TOKEN_RIGHT_ASSIGN) {
+	to_analyze=1;
+      }
+    }
+
     /* looking only for function calls here */
-    if ((token = get_token()) != '(')
+    if (!to_analyze &&
+	(token = get_token()) != '(')
     {
         if (flags & INCLUDE_ALL_REFERENCES)
             log_vulnerability(Reference, Medium);
@@ -712,18 +761,19 @@ void analyze_identifier(void)
     if (data->RaceCheck > 0)
     {
         analyzed++;
-        record_toctou(data->RaceCheck, lineno, 0);
+        record_toctou(data->RaceCheck, lineno, 0, column);
     }
     if (data->RaceUse > 0)
     {
         analyzed++;
-        record_toctou(data->RaceUse, lineno, 1);
+        record_toctou(data->RaceUse, lineno, 1, column);
     }
     if (data->Input > 0)
     {
         analyzed++;
-        if (flags & INPUT_MODE)
+        if (flags & INPUT_MODE) {
             record_input();
+	}
     }
 
     if (!analyzed)
@@ -761,7 +811,7 @@ void process_toctou(void)
     /* build a table for sorting and sort it, first by name and then by check
      * vs. use.  sort checks ahead of uses
      */
-    table = (toctou_t **)malloc(sizeof(toctou_t *) * toctou_count);
+    table = (toctou_t **)calloc(sizeof(toctou_t *) * toctou_count,1);
     for (toctou = toctous;  toctou != (toctou_t *)NULL;  toctou = toctou->next)
         table[i++] = toctou;
     qsort(table, toctou_count, sizeof(toctou_t *), toctou_sort);
@@ -796,6 +846,7 @@ void process_toctou(void)
 void setup_perl(FILE *fd)
 {
     yyperlin = fd;
+    ratslexer.lex_column = &perllex_column;
     ratslexer.yytext = &yyperltext;
     ratslexer.yyin =  fd;
     ratslexer.yylex = yyperllex;
@@ -814,6 +865,7 @@ void setup_python(FILE *fd)
     ratslexer.yycomment = &yypcomment;
     ratslexer.lex_lineno = &plex_lineno;
     ratslexer.langhash = (Hash)HashGet(database, "python");
+    ratslexer.lex_column = &plex_column;
     ratslexer.lang = LANG_PYTHON;
 }
 
@@ -826,6 +878,7 @@ void setup_c(FILE *fd)
     ratslexer.yycomment = &yyccomment;
     ratslexer.lex_lineno = &clex_lineno;
     ratslexer.langhash = (Hash)HashGet(database, "c");
+    ratslexer.lex_column = &clex_column;
     ratslexer.lang = LANG_C;
 }
 
@@ -838,10 +891,13 @@ void setup_php(FILE *fd)
     ratslexer.yycomment = &yyphpcomment;
     ratslexer.lex_lineno = &phplex_lineno;
     ratslexer.langhash = (Hash)HashGet(database, "php");
+    ratslexer.lex_column = &phplex_column;
     ratslexer.lang = LANG_PHP;
 }
 
-void determine_language(char *filename, FILE *fd, int forcelang)
+/* Changed to char type to return 1 on successful language detemrination, 0 otherwise 
+ * -- Robert */
+char determine_language(char *filename, FILE *fd, int forcelang)
 {
     char *  dot;
 
@@ -857,12 +913,16 @@ void determine_language(char *filename, FILE *fd, int forcelang)
             setup_perl(fd);
 	else if ( forcelang == LANG_PHP)
 	  setup_php(fd);
-        return;
+        return 1;
     }
     if (!dot)
     {
-        setup_c(fd);
-        return;
+      if(flags |= RECURSIVE_FILE_SCAN) {
+	/* Skip the file if there's no dot on a recurssive file scan */
+	return 0;
+      }
+      setup_c(fd);
+      return 1;
     }
     if (!strcasecmp(dot, ".py"))
         setup_python(fd);
@@ -870,26 +930,191 @@ void determine_language(char *filename, FILE *fd, int forcelang)
         setup_perl(fd);
     else if (!strcasecmp(dot, ".php"))
         setup_php(fd);
-    else
+    else if (!strcasecmp(dot, ".c")||
+	     !strcasecmp(dot, ".c++")||
+	     !strcasecmp(dot, ".cp")||
+	     !strcasecmp(dot, ".cpp")||
+	     !strcasecmp(dot, ".cc")||
+	     !strcasecmp(dot, ".cxx")||
+	     !strcasecmp(dot, ".c++")||
+	     !strcasecmp(dot, ".C")||
+	     !strcasecmp(dot, ".i")||
+	     !strcasecmp(dot, ".ii")) {
         setup_c(fd);
+    }
+    /* For now, we skip it if it's not a match -- Robert */
+    else {
+      return 0;
+    }
+    return 1;
 }
 
-void process_file(char *filename, FILE *fd, int forcelang)
+
+void process_directory(char *filename, int forcelang) {
+#ifdef _MSC_VER
+  HANDLE dir;
+  WIN32_FIND_DATA dirdata;
+  char *newfname;
+  BOOL ok;
+  int error;
+
+#else
+  DIR *dir;
+  struct dirent *dirdata;
+#endif
+
+  char *buf;
+  
+/* Due to the way MSVC handles things, it appears that we need to
+ * do the actual processing of the first file before we go into the
+ * loop */
+
+
+#ifdef _MSC_VER
+  newfname=calloc(PATH_MAX,1);
+  sprintf(newfname,"%s/*",filename);
+  for (dir=FindFirstFile(newfname, &dirdata), ok=1;
+       dir!=INVALID_HANDLE_VALUE && ok;
+       ok=FindNextFile(dir, &dirdata)) {
+    newfname=calloc(strlen(dirdata.cFileName)+1,1);
+    strcpy(newfname,dirdata.cFileName);
+    if(!strcmp(dirdata.cFileName,".") ||
+       !strcmp(dirdata.cFileName,"..") ) {
+      continue;
+    }
+    buf=calloc(PATH_MAX,1);
+    sprintf(buf,
+	    "%s/%s",
+	    filename,
+	    dirdata.cFileName);
+    process_file(buf,forcelang);
+  }
+
+  error = GetLastError();
+  if (error!=ERROR_NO_MORE_FILES) {
+    printf("Find*File: error %d\n", error);
+  }
+  
+  if (dir!=INVALID_HANDLE_VALUE) {
+    BOOL ok = FindClose(dir);
+    if (!ok) {
+      printf("FindClose: error %d", GetLastError());
+      return;
+    }
+  }
+#else
+  if((dir=opendir(filename))==NULL) {
+    fprintf(stderr,"There was a problem opening the directory.\n");
+    return;
+  }
+  while((dirdata=readdir(dir))!=NULL) {
+    if(!strcmp(dirdata->d_name,".")||
+       !strcmp(dirdata->d_name,"..")) {
+      continue;
+    }
+    buf=calloc(PATH_MAX,1);
+    sprintf(buf,
+	    "%s/%s",
+	    filename,
+	    dirdata->d_name);
+    process_file(buf,forcelang);
+  }
+#endif
+}
+
+void process_file(char *filename, int forcelang)
 {
+    FILE *fd = NULL;
     int             i;
     accumulator_t * acc;
+#ifdef _MSC_VER
+    struct _stat fstat;
+#else
+	struct stat fstat;
+#endif
+
+    /* 
+     * We need to determine the filetype first.
+     * If it's a real file, process normally.  
+     * If it's a directory and the recursion flag is set, iterate through the
+     * files in the directory and run process_file on them.
+     */
+
+    if (!strcmp(filename, "<stdin>"))
+    {
+      fd=stdin;
+    } else {
+    
+      if(lstat(filename,&fstat)==-1) 
+      {
+        fprintf(stderr,"An error occurred. The file '%s' does not appear to exist\n", filename);
+      return;
+      }
+#ifndef _MSC_VER
+      /* Symbolic link check */
+      if(S_ISLNK(fstat.st_mode)) {
+        if(flags & FOLLOW_SYMLINK) {
+	  char *symname;
+	  symname=calloc(PATH_MAX,1);
+	  if(readlink(filename,symname,PATH_MAX)==-1) {
+	    return;
+	  }
+	  process_file(symname,forcelang);
+        }
+        return;
+      }
+#endif
+      if(S_ISDIR(fstat.st_mode)) {
+        /* Need to error catch here.*/
+        if( flags & RECURSIVE_FILE_SCAN ) {
+	  process_directory(filename,forcelang);
+	  return;
+        }
+     }
+    
+     if (!S_ISREG(fstat.st_mode))
+     {
+       printf("NOT REGULAR FILE\n");
+       return;
+     } 
+
+   }
+ 
+
+   if(!fd && (fd=fopen(filename,"r")) == (FILE *)NULL) {
+      return;
+   }
 
     /* (Re-)Initialize state */
-    determine_language(filename, fd, forcelang);
+    if(!determine_language(filename, fd, forcelang)) {
+      fclose(fd);
+      return;
+    }
+
     *ratslexer.lex_lineno = 1;
     last_text_line = 0;
     current_file = strdup(filename);
     for (i = 0;  i < DEPTH_COUNT;  depths[i++] = 0);
 
     /* Process the file */
+
+   if (!(flags & NO_STATUS))
+   {
+     if ((flags & HTML_OUTPUT))
+     {
+       printf("Analyzing <b>%s</b><br>\n", filename);
+     } else if (flags & XML_OUTPUT) {
+       printf("<analyzed>%s</analyzed>\n", filename);
+     } else {
+       printf("Analyzing %s\n", filename);
+     }
+   }
+
     scan_tokens((processorfn_t)NULL, NULL);
     process_toctou();
+    
 
+    total_lines += *ratslexer.lex_lineno;
     /* Cleanup */
     current_ignore = (ignore_t *)NULL;
     while ((acc = accumulators) != (accumulator_t *)NULL)
@@ -898,6 +1123,7 @@ void process_file(char *filename, FILE *fd, int forcelang)
             free(*(acc->text));
         pop_accumulator();
     }
+    fclose(fd);
 }
 
 static
