@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2001 Secure Software Solutions
+ * Copyright (c) 2001-2002 Secure Software, Inc
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,6 +30,17 @@
 #define INCLUDE_ALL_REFERENCES  0x1
 #define INPUT_MODE              0x2
 #define RECURSIVE_FILE_SCAN     0x4
+#define XML_OUTPUT		0x8
+#define HTML_OUTPUT		0x10
+#define FOLLOW_SYMLINK		0x20
+#define NO_HEADER		0x40
+#define NO_STATUS		0x80
+#define NO_FOOTER		0x100
+#define SHOW_COLUMNS	        0x200
+#define SHOW_CONTEXT		0x400
+#define ALL_STATIC		0x800
+
+
 
 #define LANG_PYTHON		1	
 #define LANG_C		 	2	
@@ -47,6 +58,7 @@ struct _lexer_t
     int *	    lex_lineno;
     Hash	    langhash;
     int		    lang;
+    int *	    lex_column;
 };
 
     
@@ -64,6 +76,7 @@ struct _rats_stack_t
 {
     char *          identifier;
     Vuln_t *        data;
+    int		    column;
     int             lineno;
     int             argc;
     argument_t *    argv;
@@ -83,6 +96,8 @@ struct _charscan_t
 {
     int lineno;
     int last_token;
+    int column;
+    int initial_type;
     int depth;
     int skip;
 };
@@ -98,6 +113,7 @@ struct _accumulator_t
 typedef struct _toctou_t toctou_t;
 struct _toctou_t
 {
+    int		column;
     int         lineno;
     Vuln_t *    data;
     char *      key;
@@ -113,6 +129,6 @@ extern Hash	        defaultdb;
 extern char *           current_file;
 extern rats_stack_t *   current_frame;
 
-extern void process_file(char *, FILE *, int);
+extern void process_file(char *, int);
 
 #endif  /* ENGINE_H */
