@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2001 Secure Software Solutions
+ * Copyright (c) 2001-2002 Secure Software, Inc
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,42 +41,42 @@
 
 /* Tokens that are common to multiple languages */
 
-#define HEX_CONST 	(CONST_START +  0)
-#define OCT_CONST	(CONST_START +  1)
-#define DEC_CONST	(CONST_START +  2)
-#define FLOAT_CONST	(CONST_START +  3)
-#define IMAG_CONST	(CONST_START +  4)
-#define STRING_CONST    (CONST_START +  5)
-#define CHAR_CONST	(CONST_START +  6)
+#define TOKEN_HEX_CONST 	(CONST_START +  0)
+#define TOKEN_OCT_CONST	(CONST_START +  1)
+#define TOKEN_DEC_CONST	(CONST_START +  2)
+#define TOKEN_FLOAT_CONST	(CONST_START +  3)
+#define TOKEN_IMAG_CONST	(CONST_START +  4)
+#define TOKEN_STRING_CONST    (CONST_START +  5)
+#define TOKEN_CHAR_CONST	(CONST_START +  6)
 
-#define BREAK           (TOKEN_START +  0)
-#define CONTINUE        (TOKEN_START +  1)
-#define ELSE            (TOKEN_START +  2)
-#define FOR             (TOKEN_START +  3)
-#define IF              (TOKEN_START +  4)
-#define RETURN          (TOKEN_START +  5)
-#define WHILE           (TOKEN_START +  6)
-#define IDENTIFIER      (TOKEN_START + 11)
-#define COMMENT         (TOKEN_START + 13)
-#define JUNK            (TOKEN_START + 13)
+#define TOKEN_BREAK           (TOKEN_START +  0)
+#define TOKEN_CONTINUE        (TOKEN_START +  1)
+#define TOKEN_ELSE            (TOKEN_START +  2)
+#define TOKEN_FOR             (TOKEN_START +  3)
+#define TOKEN_IF              (TOKEN_START +  4)
+#define TOKEN_RETURN          (TOKEN_START +  5)
+#define TOKEN_WHILE           (TOKEN_START +  6)
+#define TOKEN_IDENTIFIER      (TOKEN_START + 11)
+#define TOKEN_COMMENT         (TOKEN_START + 13)
+#define TOKEN_JUNK            (TOKEN_START + 14)
 
-#define RIGHT_ASSIGN    (TOKEN_START + 15)
-#define LEFT_ASSIGN     (TOKEN_START + 16)
-#define ADD_ASSIGN      (TOKEN_START + 1)
-#define SUB_ASSIGN      (TOKEN_START + 18)
-#define MUL_ASSIGN      (TOKEN_START + 19)
-#define DIV_ASSIGN      (TOKEN_START + 20)
-#define MOD_ASSIGN      (TOKEN_START + 21)
-#define AND_ASSIGN      (TOKEN_START + 22)
-#define XOR_ASSIGN      (TOKEN_START + 23)
-#define OR_ASSIGN       (TOKEN_START + 24)
+#define TOKEN_RIGHT_ASSIGN    (TOKEN_START + 15)
+#define TOKEN_LEFT_ASSIGN     (TOKEN_START + 16)
+#define TOKEN_ADD_ASSIGN      (TOKEN_START + 17)
+#define TOKEN_SUB_ASSIGN      (TOKEN_START + 18)
+#define TOKEN_MUL_ASSIGN      (TOKEN_START + 19)
+#define TOKEN_DIV_ASSIGN      (TOKEN_START + 20)
+#define TOKEN_MOD_ASSIGN      (TOKEN_START + 21)
+#define TOKEN_AND_ASSIGN      (TOKEN_START + 22)
+#define TOKEN_XOR_ASSIGN      (TOKEN_START + 23)
+#define TOKEN_OR_ASSIGN       (TOKEN_START + 24)
 
-#define RIGHT_OP        (TOKEN_START + 25)
-#define LEFT_OP         (TOKEN_START + 26)
-#define LE_OP           (TOKEN_START + 27)
-#define GE_OP           (TOKEN_START + 28)
-#define EQ_OP           (TOKEN_START + 29)
-#define NE_OP           (TOKEN_START + 30)
+#define TOKEN_RIGHT_OP        (TOKEN_START + 25)
+#define TOKEN_LEFT_OP         (TOKEN_START + 26)
+#define TOKEN_LE_OP           (TOKEN_START + 27)
+#define TOKEN_GE_OP           (TOKEN_START + 28)
+#define TOKEN_EQ_OP           (TOKEN_START + 29)
+#define TOKEN_NE_OP           (TOKEN_START + 30)
 
 /* Language specific tokens */
 #include "c-tokens.h"
@@ -86,6 +86,7 @@
 
 /* Common externs */
 /*C language */
+extern int	clex_column;
 extern int      clex_lineno;
 extern FILE *   yycin;
 extern char *   yyctext;
@@ -96,6 +97,7 @@ extern char *   yyccomment;
 extern int yyclex(void);
 
 /*python language */
+extern int 	plex_column;
 extern int      plex_lineno;
 extern FILE *   yypin;
 extern char *   yyptext;
@@ -105,6 +107,7 @@ extern char *   yypcomment;
 extern int yyplex(void);
 
 /* perl language */
+extern int 	perllex_column;
 extern int 	perllex_lineno;
 extern FILE *	yyperlin;
 extern char *	yyperltext;
@@ -113,6 +116,7 @@ extern char *	yyperlcomment;
 extern int yyperllex(void);
 
 /* php language */
+extern int 	phplex_column;
 extern int      phplex_lineno;
 extern FILE *   yyphpin;
 extern char *   yyphptext;
